from marshmallow import EXCLUDE, fields, Schema

from livestyled.models.reality import Reality, RealityType
from livestyled.schemas.fields import RelatedResourceLinkField


class RealityTypeSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'reality_type'
        url = 'user_management/reality_types'
        model = RealityType

    id = fields.Int()
    name = fields.String(missing=None)
    config_json_schema = fields.Raw(missing={}, data_key='configJsonSchema', allow_none=True)
    config_ui_schema = fields.Raw(missing={}, data_key='configUiSchema', allow_none=True)
    evaluator = fields.Raw(missing={})
    watch = fields.Raw()
    value_type = fields.String(data_key='valueType')
    updated_at = fields.AwareDateTime(data_key='updatedAt', allow_none=True, missing=None)
    created_at = fields.AwareDateTime(data_key='createdAt', allow_none=True, missing=None)


class RealitySchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'realities'
        url = 'user_management/realities'
        model = Reality

    id = fields.Int()
    reality_type = RelatedResourceLinkField(schema=RealityTypeSchema, allow_none=False, data_key='realityType')
    name = fields.String(missing=None)
    config = fields.Raw(missing={})
    updated_at = fields.AwareDateTime(data_key='updatedAt', allow_none=True, missing=None)
    created_at = fields.AwareDateTime(data_key='createdAt', allow_none=True, missing=None)



from marshmallow import EXCLUDE, fields, Schema

from livestyled.models.audience import Audience
from livestyled.schemas.fields import RelatedResourceLinkField
from livestyled.schemas.reality import RealitySchema


class AudienceRealityValuesValueSchema(Schema):
    operator = fields.String()
    value = fields.String()


class AudienceRealityValuesSchema(Schema):
    reality = RelatedResourceLinkField(schema=RealitySchema)
    values = fields.Nested(AudienceRealityValuesValueSchema, many=True)


class AudienceSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'audiences'
        url = 'user_management/audiences'
        model = Audience

    id = fields.Int()
    name = fields.String(missing=None)
    reality_values = fields.Nested(AudienceRealityValuesSchema, many=True, data_key='realityValues')
    updated_at = fields.AwareDateTime(data_key='updatedAt', allow_none=True, missing=None)
    created_at = fields.AwareDateTime(data_key='createdAt', allow_none=True, missing=None)



import os

from livestyled.models import FulfilmentPoint, FulfilmentPointCategory, FulfilmentPointTranslation
from livestyled.resource_client import LiveStyledResourceClient
from livestyled.tests.utils import configure_mock_responses

FIXTURES_DIR = os.path.join(os.path.dirname(__file__), 'fixtures')
TEST_API_DOMAIN = 'test.livestyled.com'
CONTENT_TYPE = 'application/ld+json'


def test_get_fulfilment_points(requests_mock):
    mock_responses = (
        ('GET', 'https://' + TEST_API_DOMAIN + '/v4/sell/fulfilment_points', 'mock_responses/ls_api/fulfilment_points.json', 200),
    )
    configure_mock_responses(requests_mock, mock_responses, FIXTURES_DIR, CONTENT_TYPE)

    resource_client = LiveStyledResourceClient(TEST_API_DOMAIN, 'bar')
    fps = list(resource_client.get_fulfilment_points())
    assert len(fps) == 30
    fps.sort(key=lambda o: o.id)
    fp = fps[0]
    assert isinstance(fp, FulfilmentPoint)
    assert fp.id == 1
    assert fp.status == 'ACTIVE'
    assert fp.reference == 'kartiks Collection point'
    assert fp.image_url == 'https://apiv3.s3.eu-west-1.amazonaws.com/fulfilmentpoint/German-Shepherd-Puppy-Fetch.jpg'
    assert fp.map_image_url == 'https://apiv3.s3.eu-west-1.amazonaws.com/fulfilmentpoint/british_summer_time_map_bigger.jpg'
    assert fp.lat == '38.026641'
    assert fp.lat == '38.026641'
    assert fp.type == 'VIRTUAL_QUEUE'
    assert fp.position == 0
    assert isinstance(fp.translations, list)
    assert isinstance(fp.translations[0], FulfilmentPointTranslation)
    assert fp.translations[0].language == 'en'
    assert fp.translations[0].title == 'Fulfilment point 1'
    assert fp.translations[0].description == 'Description'
    assert fp.translations[0].collection_note == 'Fulfillment note 1'
    assert isinstance(fp.categories, list)
    assert isinstance(fp.categories[0], FulfilmentPointCategory)



from datetime import datetime
from typing import Dict


class RealityType:
    def __init__(
            self,
            id,
            name,
            config_json_schema,
            config_ui_schema,
            evaluator,
            watch,
            value_type,
            updated_at,
            created_at
    ):
        self.id = id
        self.name = name
        self.config_json_schema = config_json_schema
        self.config_ui_schema = config_ui_schema
        self.evaluator = evaluator
        self.watch = watch
        self.value_type = value_type
        self.updated_at = updated_at
        self.created_at = created_at

    @classmethod
    def placeholder(cls, id):
        return cls(
            id,
            name=None,
            config_json_schema=None,
            config_ui_schema=None,
            evaluator=None,
            watch=None,
            value_type=None,
            updated_at=None,
            created_at=None
        )

    def diff(self, other):
        differences = {}
        fields = (
            'name', 'config_json_schema', 'config_ui_schema', 'evaluator', 'watch', 'updated_at', 'created_at'
        )
        for field in fields:
            if getattr(self, field) != getattr(other, field):
                differences[field] = getattr(self, field)
        return differences


class Reality:
    def __init__(
            self,
            id,
            reality_type: RealityType,
            name: str,
            config: Dict,
            updated_at: datetime,
            created_at: datetime
    ):
        self.id = id
        self.name = name
        self.created_at = created_at
        self.updated_at = updated_at
        self.config = config

        if reality_type:
            if isinstance(reality_type, RealityType):
                self.reality_type = reality_type
            elif isinstance(reality_type, dict):
                self.reality_type = RealityType(**reality_type)
            elif isinstance(reality_type, int):
                self.reality_type = RealityType.placeholder(id=reality_type)
        else:
            self.reality_type = None

    @classmethod
    def placeholder(cls, id):
        return cls(
            id,
            reality_type=None,
            name=None,
            config={},
            updated_at=None,
            created_at=None
        )

    def diff(self, other):
        differences = {}
        fields = (
            'reality_type', 'name', 'config'
        )
        for field in fields:
            if getattr(self, field) != getattr(other, field):
                differences[field] = getattr(self, field)
        return differences



from datetime import datetime
from typing import Dict

from livestyled.models.audience import Audience
from livestyled.models.device import Device


class AudienceDevice:
    def __init__(
            self,
            audience: Dict or Audience,
            device: Dict or Device,
            created_at: datetime or None = None
    ):
        self.created_at = created_at
        if isinstance(audience, Audience):
            self.audience = audience
        elif isinstance(audience, int):
            self.audience = Audience.placeholder(id=audience)
        else:
            self.audience = Audience(**audience)
        if isinstance(device, Device):
            self.device = device
        elif isinstance(device, int):
            self.device = Device.placeholder(id=device)
        else:
            self.device = Device(**device)

    @property
    def id(self):
        return 'device={};audience={}'.format(self.device.id, self.audience.id)

    @property
    def compound_id(self):
        return True



from datetime import datetime
from typing import Dict, List

from livestyled.models.reality import Reality


class AudienceRealityValueValue:
    def __init__(
            self,
            operator: str,
            value: str
    ):
        self.operator = operator
        self.value = value


class AudienceRealityValue:
    def __init__(
            self,
            reality: Dict,
            values: List[Dict]
    ):
        self.reality = Reality.placeholder(reality)
        self.values = []
        for value in values:
            self.values.append(AudienceRealityValueValue(**value))


class Audience:
    def __init__(
            self,
            id,
            name: str,
            reality_values: List[Dict],
            updated_at: datetime,
            created_at: datetime
    ):
        self.id = id
        self.name = name
        self.created_at = created_at
        self.updated_at = updated_at

        self.reality_values = []
        for reality_value in reality_values:
            self.reality_values.append(AudienceRealityValue(**reality_value))

    @classmethod
    def placeholder(cls, id):
        return cls(
            id,
            name=None,
            reality_values=[],
            updated_at=None,
            created_at=None
        )



from datetime import datetime

from livestyled.models.device import Device
from livestyled.models.reality import Reality


class DeviceReality:
    def __init__(
            self,
            id,
            device: Device or str,
            reality: Reality or str,
            value: str,
            created_at: datetime,
            updated_at: datetime,
    ):
        self.id = id
        self.value = value
        self.created_at = created_at
        self.updated_at = updated_at

        if device:
            if isinstance(device, Device):
                self.device = device
            elif isinstance(device, dict):
                self.device = Device(**device)
            elif isinstance(device, int):
                self.device = Device.placeholder(id=device)
        else:
            self.device = None

        if reality:
            if isinstance(reality, Reality):
                self.reality = reality
            elif isinstance(reality, dict):
                self.reality = Reality(**reality)
            elif isinstance(reality, int):
                self.reality = Reality.placeholder(id=reality)
        else:
            self.reality = None

    @classmethod
    def placeholder(cls, id):
        return cls(
            id,
            device=None,
            reality=None,
            value=None,
            created_at=None,
            updated_at=None
        )

    @classmethod
    def create_new(
            cls,
            device: Device,
            reality: Reality,
            value: str
    ):
        device_reality = DeviceReality(
            id=None,
            device=device,
            reality=reality,
            value=value,
            created_at=None,
            updated_at=None,
        )
        return device_reality

    def diff(self, other):
        differences = {}
        fields = (
            'value'
        )
        for field in fields:
            if getattr(self, field) != getattr(other, field):
                differences[field] = getattr(self, field)
        return differences



import json
import os

from livestyled.models import Product
from livestyled.resource_client import LiveStyledResourceClient
from livestyled.tests.utils import configure_mock_responses

FIXTURES_DIR = os.path.join(os.path.dirname(__file__), 'fixtures')
TEST_API_DOMAIN = 'test.livestyled.com'
CONTENT_TYPE = 'application/ld+json'


def test_get_products(requests_mock):
    mock_responses = (
        ('GET', 'https://' + TEST_API_DOMAIN + '/v4/sell/products', 'mock_responses/ls_api/products.json', 200),
    )
    configure_mock_responses(requests_mock, mock_responses, FIXTURES_DIR, CONTENT_TYPE)

    resource_client = LiveStyledResourceClient(TEST_API_DOMAIN, 'bar')
    products = list(resource_client.get_products())
    assert len(products) == 30
    products.sort(key=lambda o: o.id)
    product = products[2]

    assert isinstance(product, Product)
    assert product.id == 3
    assert product.status == 'ACTIVE'
    assert product.reference == "Men's Tank"
    assert len(product.images) == 1
    assert isinstance(product.images[0], dict)
    assert product.images[0] == {
        'external_id': '6N6H3B3IZVGBK3OMFOCFNGE2',
        'image_url': 'https://apiv3.s3.eu-west-1.amazonaws.com/productimage/original-2.jpeg',
        'position': 1
    }
    assert len(product.translations) == 1
    assert product.translations[0]['title'] == "Men's Tank"
    assert product.translations[0]['description'] is None
    assert product.translations[0]['language'] == 'en'


def test_create_product(requests_mock):
    resource_client = LiveStyledResourceClient(TEST_API_DOMAIN, 'bar')
    mock_responses = (
        ('POST', 'https://' + TEST_API_DOMAIN + '/v4/sell/products', 'mock_responses/ls_api/sell/new_product.json', 200),
    )
    configure_mock_responses(requests_mock, mock_responses, FIXTURES_DIR, CONTENT_TYPE)

    resource_client.create_product(
        Product.create_new(
            external_id='testproduct1',
            categories=None,
            status='ACTIVE',
            reference='Test Product 1',
            translations=[
                {
                    'language': 'en',
                    'title': 'Test Product 1',
                    'description': 'This is a Test Product'
                }
            ],
            price=100,
            fulfilment_points=[],
            core_product_category=None,
            holding_time=None,
            images=None,
            modifier_lists=None,
            reconciliation_group=None,
            variants=None,
        )
    )

    assert len(requests_mock.request_history) == 1
    assert requests_mock.request_history[0].method == 'POST'
    assert requests_mock.request_history[0].url == 'https://' + TEST_API_DOMAIN + '/v4/sell/products'
    assert json.loads(requests_mock.request_history[0].body) == {
        'externalId': 'testproduct1',
        'reference': 'Test Product 1',
        'status': 'ACTIVE',
        'translations': [
            {
                'description': 'This is a Test Product',
                'language': 'en',
                'title': 'Test Product 1'
            }
        ],
        'images': [],
        'categories': [],
        'fulfilmentPoints': [],
        'modifierLists': [],
        'variants': []
    }



from marshmallow import EXCLUDE, fields, Schema

from livestyled.models.audience_device import AudienceDevice
from livestyled.schemas.fields import RelatedResourceLinkField
from livestyled.schemas.audience import AudienceSchema
from livestyled.schemas.device import DeviceSchema


class AudienceDeviceSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'audience_devices'
        url = 'user_management/audience_devices'
        model = AudienceDevice

    audience = RelatedResourceLinkField(schema=AudienceSchema)
    device = RelatedResourceLinkField(schema=DeviceSchema)
    created_at = fields.AwareDateTime(data_key='createdAt', allow_none=True, missing=None)



from marshmallow import EXCLUDE, fields, Schema

from livestyled.models.device_reality import DeviceReality
from livestyled.schemas.device import DeviceSchema
from livestyled.schemas.fields import RelatedResourceLinkField
from livestyled.schemas.reality import RealitySchema


class DeviceRealitySchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'device_realities'
        url = 'user_management/device_realities'
        model = DeviceReality

    id = fields.Int()
    reality = RelatedResourceLinkField(schema=RealitySchema, allow_none=False)
    device = RelatedResourceLinkField(schema=DeviceSchema, allow_none=False)
    value = fields.String(missing=None)
    updated_at = fields.AwareDateTime(data_key='updatedAt', allow_none=True, missing=None)
    created_at = fields.AwareDateTime(data_key='createdAt', allow_none=True, missing=None)



from marshmallow import EXCLUDE, fields, Schema

from livestyled.models.product import (
    Product,
    ProductCategory,
    ProductModifierItem,
    ProductModifierList,
    ProductVariant
)
from livestyled.schemas.fields import RelatedResourceField, RelatedResourceLinkField
from livestyled.schemas.fulfilment_point import FulfilmentPointSchema


class ProductVariantStocksSchema(Schema):
    class Meta:
        unknown = EXCLUDE

    id = fields.Integer()


class ProductVariantSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        model = ProductVariant
        url = 'sell/product_variants'

    class ProductVariantTranslationSchema(Schema):
        class Meta:
            unknown = EXCLUDE
        language = fields.String()
        title = fields.String()

    id = fields.Int()
    price = fields.Integer()
    stocks = RelatedResourceLinkField(schema=ProductVariantStocksSchema, many=True, missing=[])
    product = RelatedResourceLinkField(schema='livestyled.schemas.product.ProductSchema')
    external_id = fields.String(missing=None, data_key='externalId')
    translations = fields.Nested(ProductVariantTranslationSchema, many=True, missing=None)


class ProductCategorySchema(Schema):
    class Meta:
        unknown = EXCLUDE
        model = ProductCategory
        api_type = 'product_categories'
        url = 'sell/product_categories'

    class ProductCategoryTranslationSchema(Schema):
        class Meta:
            unknown = EXCLUDE
        language = fields.String()
        title = fields.String()

    id = fields.Int()
    reference = fields.String(missing=None)
    position = fields.Int()
    external_id = fields.String(missing=None, data_key='externalId')
    status = fields.String(missing=None)
    translations = fields.Nested(ProductCategoryTranslationSchema, many=True, missing=None)


class ProductImageSchema(Schema):
    class Meta:
        unknown = EXCLUDE

    position = fields.Integer()
    image_url = fields.String(data_key='imageUrl', missing=None)
    external_id = fields.String(data_key='externalId', missing=None)


class ProductTranslationSchema(Schema):
    class Meta:
        unknown = EXCLUDE

    language = fields.String()
    title = fields.String(missing=None)
    description = fields.String(missing=None)


class CoreProductCategorySchema(Schema):
    class Meta:
        unknown = EXCLUDE
    id = fields.Integer()
    name = fields.String()


class ProductModifierItemTranslationsSchema(Schema):
    class Meta:
        unknown = EXCLUDE

    language = fields.String()
    title = fields.String(missing=None)


class ProductModifierItemSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        model = ProductModifierItem
        url = 'sell/product_modifier_items'

    id = fields.Int()
    external_id = fields.String(data_key='externalId')
    additional_price = fields.Integer(data_key='additionalPrice')
    translations = fields.Nested(ProductModifierItemTranslationsSchema, many=True, missing=[])
    modifier_list = RelatedResourceLinkField('livestyled.schemas.product.ProductModifierListSchema', data_key='modifierList')


class ProductModifierListTranslationsSchema(Schema):
    class Meta:
        unknown = EXCLUDE

    language = fields.String()
    title = fields.String(missing=None)


class ProductModifierListSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        model = ProductModifierList
        url = 'sell/product_modifier_lists'

    id = fields.Integer()
    status = fields.String()
    external_id = fields.String(data_key='externalId')
    reference = fields.String(missing=None)
    multiple_select = fields.Boolean(data_key='multipleSelect')
    mandatory_select = fields.Boolean(data_key='mandatorySelect')
    translations = fields.Nested(ProductModifierListTranslationsSchema, many=True, missing=[])
    items = RelatedResourceField(schema=ProductModifierItemSchema, many=True, missing=[])


class ProductSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        model = Product
        api_type = 'products'
        url = 'sell/products'

    id = fields.Int()
    status = fields.String(missing=None)
    reference = fields.String(missing=None)
    modifier_lists = RelatedResourceField(schema=ProductModifierListSchema, many=True, missing=[], data_key='modifierLists')
    external_id = fields.String(data_key='externalId', missing=None)
    holding_time = fields.Raw(missing=None, data_key='holdingTime')
    reconciliation_group = fields.Raw(missing=None, data_key='reconciliation_group')
    images = fields.Nested(ProductImageSchema, many=True)
    categories = RelatedResourceField(schema=ProductCategorySchema, many=True, missing=[])
    translations = fields.Nested(ProductTranslationSchema, many=True, missing=[])
    fulfilment_points = RelatedResourceField(schema=FulfilmentPointSchema, many=True, missing=[], data_key='fulfilmentPoints')
    variants = RelatedResourceField(schema=ProductVariantSchema, many=True, missing=[])
    core_product_category = fields.Nested(CoreProductCategorySchema, missing=None, data_key='coreProductCategory')
